@@ -2,6 +2,7 @@ import json
 import queue
 import re
 import os
+import sys
 import time
 import traceback
 from pathlib import Path
@@ -29,8 +30,10 @@ warnings.simplefilter("ignore")
 
 
 requests.packages.urllib3.disable_warnings(category=InsecureRequestWarning)
-
-logging.basicConfig(format='%(asctime)s : %(message)s', filename='dyagram.log', filemode='a')
+file_handler = logging.FileHandler('dyagram.log')
+stdout_handler = logging.StreamHandler(stream=sys.stdout)
+handlers = [file_handler, stdout_handler]
+logging.basicConfig(format='%(asctime)s : %(message)s', handlers=handlers)
 
 
 class Dyagram:
@@ -75,6 +78,7 @@ class Dyagram:
     def setup_logging(self):
         log = logging.getLogger("")
         self.log = log
+
         if self.verbose:
             self.log.setLevel(logging.INFO)
         else:
@@ -387,127 +391,131 @@ class Dyagram:
 
     @staticmethod
     def get_state_diff(previous_state, current_state, log=None):
-        log.info(f"STARTING GET_STATE_DIFF")
-        device_diffs = {}
+        try:
+            log.info(f"STARTING GET_STATE_DIFF")
+            device_diffs = {}
 
-        elem_changed = {
-            "routes": []}  # this keeps up with "changes" when converting to removed/add for routes,
-                           # this is due to deeper dictionary items
-        log.info(f"BEFORE DeepDiff")
-        diff_resp = DeepDiff(previous_state, current_state, ignore_order=True)
-        log.info(f"AFTER DeepDiff")
-        log.info(f"diff_resp = {diff_resp}")
-        if not diff_resp:
-            return None
+            elem_changed = {
+                "routes": []}  # this keeps up with "changes" when converting to removed/add for routes,
+                               # this is due to deeper dictionary items
+            print(f"BEFORE DeepDiff")
+            diff_resp = DeepDiff(previous_state, current_state, ignore_order=True)
+            print(f"AFTER DeepDiff")
+            print(f"diff_resp = {diff_resp}")
+            if not diff_resp:
+                return None
 
-        diff_type = None
-        for k in diff_resp.keys():
-            if "added" in k:
-                diff_type = "added"
-                s = current_state
-            elif "removed" in k:
-                diff_type = "removed"
-                s = previous_state
-            elif "changed" in k:
-                s = current_state
-                diff_type = "changed"
+            diff_type = None
+            for k in diff_resp.keys():
+                if "added" in k:
+                    diff_type = "added"
+                    s = current_state
+                elif "removed" in k:
+                    diff_type = "removed"
+                    s = previous_state
+                elif "changed" in k:
+                    s = current_state
+                    diff_type = "changed"
 
-            for diff in diff_resp[k]:
+                for diff in diff_resp[k]:
 
-                resp = re.search("root\['devices'\]\[(\d+)\]\[([^\]]*)\]", diff)
-                device_elem = int(resp.group(1))
-                category = resp.group(2).replace("'", "").replace('"', '')
-                if s['devices'][device_elem]['inventory_ip'] not in device_diffs.keys():
-                    device_diffs[s['devices'][device_elem]['inventory_ip']] = {"added": [], "removed": [],
-                                                                               "changed": []}
+                    resp = re.search("root\['devices'\]\[(\d+)\]\[([^\]]*)\]", diff)
+                    device_elem = int(resp.group(1))
+                    category = resp.group(2).replace("'", "").replace('"', '')
+                    if s['devices'][device_elem]['inventory_ip'] not in device_diffs.keys():
+                        device_diffs[s['devices'][device_elem]['inventory_ip']] = {"added": [], "removed": [],
+                                                                                   "changed": []}
 
 
-                if category == "dynamic_routing_neighbors":
-                    if 'eigrp' in diff:
-                        routing_protocol = "EIGRP"
-                    elif 'ospf' in diff:
-                        routing_protocol = "OSPF"
-                    elif 'bgp' in diff:
-                        routing_protocol = "BGP"
+                    if category == "dynamic_routing_neighbors":
+                        if 'eigrp' in diff:
+                            routing_protocol = "EIGRP"
+                        elif 'ospf' in diff:
+                            routing_protocol = "OSPF"
+                        elif 'bgp' in diff:
+                            routing_protocol = "BGP"
 
-                    if diff_type != "changed":
-                        device_diffs[s['devices'][device_elem]['inventory_ip']][diff_type].append(
-                            f"{routing_protocol} Neighbor {diff_resp[k][diff]}")
-                    else:
-                        device_diffs[s['devices'][device_elem]['inventory_ip']]["removed"].append(
-                            f"{routing_protocol} Neighbor {diff_resp[k][diff]['old_value']}")
-                        device_diffs[s['devices'][device_elem]['inventory_ip']]["added"].append(
-                            f"{routing_protocol} Neighbor {diff_resp[k][diff]['new_value']}")
-
-                elif category == "layer2":
-                    if diff_type != "changed":
-                        device_diffs[s['devices'][device_elem]['inventory_ip']][diff_type].append(
-                            f"LLDP Neighbor {diff_resp[k][diff]['hostname']} (Chassis ID: {diff_resp[k][diff]['chassis_id']})")
-
-                    else:
-                        resp = re.search("\['neighbors'\]\[(\d+)\]\['(.*)'\]", diff)
-                        neigh_elem = int(resp.group(1))
-                        neigh_property = resp.group(2)
-                        device_diffs[s['devices'][device_elem]['inventory_ip']][diff_type].append(
-                            f"LLDP Neighbor {s['devices'][device_elem][category]['neighbors'][neigh_elem]['hostname']}: {neigh_property}: "
-                            f"NEW VALUE: {diff_resp[k][diff]['new_value']}, OLD VALUE: {diff_resp[k][diff]['old_value']}")
-                elif category == "routes":
-
-                    if diff_type != "changed":
-
-                        try:
+                        if diff_type != "changed":
                             device_diffs[s['devices'][device_elem]['inventory_ip']][diff_type].append(
+                                f"{routing_protocol} Neighbor {diff_resp[k][diff]}")
+                        else:
+                            device_diffs[s['devices'][device_elem]['inventory_ip']]["removed"].append(
+                                f"{routing_protocol} Neighbor {diff_resp[k][diff]['old_value']}")
+                            device_diffs[s['devices'][device_elem]['inventory_ip']]["added"].append(
+                                f"{routing_protocol} Neighbor {diff_resp[k][diff]['new_value']}")
 
-                                f"Route {diff_resp[k][diff]['network']}/{diff_resp[k][diff]['mask']} {diff_resp[k][diff]['next_hop']}")
-                        except KeyError:
-                            # different return for next hop for XR
+                    elif category == "layer2":
+                        if diff_type != "changed":
                             device_diffs[s['devices'][device_elem]['inventory_ip']][diff_type].append(
+                                f"LLDP Neighbor {diff_resp[k][diff]['hostname']} (Chassis ID: {diff_resp[k][diff]['chassis_id']})")
 
-                                f"Route {diff_resp[k][diff]['network']}/{diff_resp[k][diff]['mask']} {diff_resp[k][diff]['nexthop_ip']}")
-                    else:
+                        else:
+                            resp = re.search("\['neighbors'\]\[(\d+)\]\['(.*)'\]", diff)
+                            neigh_elem = int(resp.group(1))
+                            neigh_property = resp.group(2)
+                            device_diffs[s['devices'][device_elem]['inventory_ip']][diff_type].append(
+                                f"LLDP Neighbor {s['devices'][device_elem][category]['neighbors'][neigh_elem]['hostname']}: {neigh_property}: "
+                                f"NEW VALUE: {diff_resp[k][diff]['new_value']}, OLD VALUE: {diff_resp[k][diff]['old_value']}")
+                    elif category == "routes":
 
-                        resp = re.search("\['routes'\]\[(\d+)\]\['(.*)'\]", diff)
-
-                        route_elem = int(resp.group(1))
-
-                        if route_elem not in elem_changed['routes']:
-                            elem_changed['routes'].append(route_elem)
+                        if diff_type != "changed":
 
                             try:
+                                device_diffs[s['devices'][device_elem]['inventory_ip']][diff_type].append(
 
-                                device_diffs[s['devices'][device_elem]['inventory_ip']]["removed"].append(
-                                    f"Route {s['devices'][device_elem][category][route_elem]['network']}/"
-        
-                                    f"{previous_state['devices'][device_elem][category][route_elem]['mask']} "
-        
-                                    f"{previous_state['devices'][device_elem][category][route_elem]['next_hop']}")
-
-                                device_diffs[s['devices'][device_elem]['inventory_ip']]["added"].append(
-
-                                    f"Route {s['devices'][device_elem][category][route_elem]['network']}/"
-        
-                                    f"{current_state['devices'][device_elem][category][route_elem]['mask']} "
-        
-                                    f"{current_state['devices'][device_elem][category][route_elem]['next_hop']}")
+                                    f"Route {diff_resp[k][diff]['network']}/{diff_resp[k][diff]['mask']} {diff_resp[k][diff]['next_hop']}")
                             except KeyError:
                                 # different return for next hop for XR
-                                device_diffs[s['devices'][device_elem]['inventory_ip']]["removed"].append(
-                                    f"Route {s['devices'][device_elem][category][route_elem]['network']}/"
+                                device_diffs[s['devices'][device_elem]['inventory_ip']][diff_type].append(
 
-                                    f"{previous_state['devices'][device_elem][category][route_elem]['mask']} "
+                                    f"Route {diff_resp[k][diff]['network']}/{diff_resp[k][diff]['mask']} {diff_resp[k][diff]['nexthop_ip']}")
+                        else:
 
-                                    f"{previous_state['devices'][device_elem][category][route_elem]['nexthop_ip']}")
+                            resp = re.search("\['routes'\]\[(\d+)\]\['(.*)'\]", diff)
 
-                                device_diffs[s['devices'][device_elem]['inventory_ip']]["added"].append(
+                            route_elem = int(resp.group(1))
 
-                                    f"Route {s['devices'][device_elem][category][route_elem]['network']}/"
+                            if route_elem not in elem_changed['routes']:
+                                elem_changed['routes'].append(route_elem)
 
-                                    f"{current_state['devices'][device_elem][category][route_elem]['mask']} "
+                                try:
 
-                                    f"{current_state['devices'][device_elem][category][route_elem]['nexthop_ip']}")
+                                    device_diffs[s['devices'][device_elem]['inventory_ip']]["removed"].append(
+                                        f"Route {s['devices'][device_elem][category][route_elem]['network']}/"
+            
+                                        f"{previous_state['devices'][device_elem][category][route_elem]['mask']} "
+            
+                                        f"{previous_state['devices'][device_elem][category][route_elem]['next_hop']}")
 
-        log.info(f"END OF GET_STATE_DIFF")
-        return device_diffs
+                                    device_diffs[s['devices'][device_elem]['inventory_ip']]["added"].append(
+
+                                        f"Route {s['devices'][device_elem][category][route_elem]['network']}/"
+            
+                                        f"{current_state['devices'][device_elem][category][route_elem]['mask']} "
+            
+                                        f"{current_state['devices'][device_elem][category][route_elem]['next_hop']}")
+                                except KeyError:
+                                    # different return for next hop for XR
+                                    device_diffs[s['devices'][device_elem]['inventory_ip']]["removed"].append(
+                                        f"Route {s['devices'][device_elem][category][route_elem]['network']}/"
+    
+                                        f"{previous_state['devices'][device_elem][category][route_elem]['mask']} "
+    
+                                        f"{previous_state['devices'][device_elem][category][route_elem]['nexthop_ip']}")
+
+                                    device_diffs[s['devices'][device_elem]['inventory_ip']]["added"].append(
+
+                                        f"Route {s['devices'][device_elem][category][route_elem]['network']}/"
+    
+                                        f"{current_state['devices'][device_elem][category][route_elem]['mask']} "
+    
+                                        f"{current_state['devices'][device_elem][category][route_elem]['nexthop_ip']}")
+
+            log.info(f"END OF GET_STATE_DIFF")
+            return device_diffs
+        except Exception as e:
+            print(f"diff_resp = {diff_resp}")
+            raise e
 
     def get_device_type(self, device):
 
